#include "Arduino.h"
#include "encoder.h"
#include "lcd.h"
#include "trx_manager.h"

int Encoder::CLK;
int Encoder::DT;
int Encoder::SW;
int Encoder::PW;
int Encoder::enc_ALast;
int Encoder::enc_AVal;
int32_t Encoder::last_micros_clickdebouncer;
int32_t Encoder::last_micros_rotatedebouncer;
byte Encoder::menu_control_current;

void Encoder::Init(int _clk, int _dt, int _sw, int _pw)
{
  menu_control_current = MENU_FREQ_HZ;
  
  CLK = _clk;
  DT = _dt;
  SW = _sw;
  PW = _pw;
  pinMode (PW, OUTPUT);
  digitalWrite(PW, 1);
  pinMode (DT, INPUT);
  pinMode (CLK, INPUT);
  pinMode (SW, INPUT_PULLUP);
  enc_ALast = digitalRead(CLK);

  attachInterrupt(SW, &Encoder::checkClick, FALLING);
  attachInterrupt(CLK, &Encoder::checkRotate, CHANGE);
}

void Encoder::setCurrentIndex(byte _index)
{
  menu_control_current = _index;
  if (menu_control_current > MENU_COUNT) menu_control_current = 1;
  if (menu_control_current < 1) menu_control_current = 1;
  LCD::displayFreqInfo();
}

byte Encoder::getCurrentIndex()
{
  return menu_control_current;
}

void Encoder::checkRotate() {
  detachInterrupt(CLK);
  enc_AVal = digitalRead(CLK);
  if (enc_AVal != enc_ALast) { // проверка на изменение значения на выводе А по сравнению с предыдущим запомненным, что означает, что вал повернулся
    // а чтобы определить направление вращения, нам понадобится вывод В.
    if (digitalRead(DT) != enc_AVal) {  // Если вывод A изменился первым - вращение по часовой стрелке
      EncoderRotated(1);
    } else {// иначе B изменил свое состояние первым - вращение против часовой стрелки
      EncoderRotated(-1);
    }
  }
  enc_ALast = enc_AVal;
  attachInterrupt(CLK, &Encoder::checkRotate, CHANGE);
}

void Encoder::checkClick() { //смена разрядности переключения частоты валкодером
  if (digitalRead(SW) == HIGH) return;
  detachInterrupt(SW);

  if ((int32_t)(micros() - last_micros_clickdebouncer) >= 500000) { //защита от дребезга контактов
    if (digitalRead(SW) == LOW)
    {
      setCurrentIndex(menu_control_current + 1);
    }
    last_micros_clickdebouncer = micros();
  }
  attachInterrupt(SW, &Encoder::checkClick, FALLING);
}

void Encoder::EncoderRotated(int direction) //энкодер повернули, здесь обработчик, direction -1 - влево, 1 - вправо
{
  int mode_new = 0;

  switch (menu_control_current) {
    case MENU_FREQ_HZ:
      TRX_Manager::setFrequency(TRX_Manager::getFrequency() + 50 * direction);
      break;
    case MENU_FREQ_KHZ:
      TRX_Manager::setFrequency(TRX_Manager::getFrequency() + 1000 * direction);
      break;
    case MENU_FREQ_MHZ:
      TRX_Manager::setFrequency(TRX_Manager::getFrequency() + 1000000 * direction);
      break;
    default:
      break;
  }
}
