#include "Arduino.h"
#include "fft.h"
#include "lcd.h"

uint32_t FFT::fpgadata_sync_counter;
byte FFT::fpgadata_stage;
uint32_t FFT::fpga_packet;
int32_t FFT::fpga_audio_stream;

arduinoFFT FFT::libFFT;
const uint16_t FFT::fft_samples;
const uint16_t FFT::fpga_sample_rate;
double FFT::fpga_audio_buffer[fft_samples];
double FFT::vReal[fft_samples];
double FFT::vImag[fft_samples];
double FFT::maxfft_signal;

void FFT::Init()
{
  //шина данных ARDUIBNO-FPGA
  pinMode (FPGA_DATA_CLK, INPUT); //clock
  pinMode (FPGA_DATA_SYNC, INPUT); //sync
  pinMode (FPGA_DATA_D0, INPUT); //data
  pinMode (FPGA_DATA_D1, INPUT); //data
  pinMode (FPGA_DATA_D2, INPUT); //data
  pinMode (FPGA_DATA_D3, INPUT); //data
  pinMode (FPGA_DATA_D4, INPUT); //data
  pinMode (FPGA_DATA_D5, INPUT); //data
  pinMode (FPGA_DATA_D6, INPUT); //data
  pinMode (FPGA_DATA_D7, INPUT); //data

  fpgadata_sync_counter = 0;
  fpgadata_stage = 0;
  fpga_packet = 0;
  fpga_audio_stream = 0;
  maxfft_signal = MIN_FFT_THRESHOLD;
  libFFT = arduinoFFT();

  attachInterrupt(FPGA_DATA_CLK, fpgadata_clock, RISING);
}

void FFT::generate_fft()
{
  int i;
  for (i = 0; i < fft_samples; i++)
  {
    vReal[i] = fpga_audio_buffer[i];
    vImag[i] = 0;
  }
  libFFT.Windowing(vReal, fft_samples, FFT_WIN_TYP_HAMMING, FFT_FORWARD);  // Weigh data
  libFFT.Compute(vReal, vImag, fft_samples, FFT_FORWARD); // Compute FFT
  libFFT.ComplexToMagnitude(vReal, vImag, fft_samples); // Compute magnitudes
  PrintFFT(vReal, (fft_samples >> 1));
  //double x = libFFT.MajorPeak(vReal, fft_samples, 12500);
  //Serial.println(x, 6);
}

void FFT::PrintFFT(double *vData, uint16_t bufferSize)
{
  /*
    for (uint16_t i = 0; i < bufferSize; i++)
    {
    double abscissa;
    abscissa = ((i * 1.0 * fpga_sample_rate) / fft_samples); //samplingFrequency = 12500;
    Serial.print(abscissa, 6);
    Serial.print("Hz");
    Serial.print(" ");
    Serial.println(vData[i], 4);
    }
  */
  maxfft_signal = MIN_FFT_THRESHOLD;
  for (uint16_t i = 0; i < fft_samples / 4; i++)
  {
    if (vData[i] > maxfft_signal)
      maxfft_signal = vData[i];
  }

  byte result[fft_samples / 4];
  for (uint16_t i = 0; i < fft_samples / 4; i++)
  {
    result[i] = vData[i] * 100 / maxfft_signal;
    if (result[i] > 100) result[i] = 100;
    //Serial.println(result[i]);
  }
  //Serial.println();
  LCD::displayFFT(result, fft_samples / 4);
}

void FFT::fpgadata_clock()
{
  if (digitalReadDirect(FPGA_DATA_SYNC))
  {
    fpgadata_sync_counter++;
    fpgadata_stage = 1;
  }
  else
    fpgadata_stage++;

  fpga_packet = digitalReadDirect(FPGA_DATA_D0);
  fpga_packet = (fpga_packet << 1) | digitalReadDirect(FPGA_DATA_D1);
  fpga_packet = (fpga_packet << 1) | digitalReadDirect(FPGA_DATA_D2);
  fpga_packet = (fpga_packet << 1) | digitalReadDirect(FPGA_DATA_D3);
  fpga_packet = (fpga_packet << 1) | digitalReadDirect(FPGA_DATA_D4);
  fpga_packet = (fpga_packet << 1) | digitalReadDirect(FPGA_DATA_D5);
  fpga_packet = (fpga_packet << 1) | digitalReadDirect(FPGA_DATA_D6);
  fpga_packet = (fpga_packet << 1) | digitalReadDirect(FPGA_DATA_D7);

  if (fpgadata_stage == 1) fpga_audio_stream = fpga_packet;
  if (fpgadata_stage == 2) fpga_audio_stream = (fpga_audio_stream << 8) | fpga_packet;
  if (fpgadata_stage == 3)
  {
    fpga_audio_stream = (fpga_audio_stream << 8) | fpga_packet;
    fpga_audio_stream = fpga_audio_stream << 8;
    fpga_audio_buffer[fpgadata_sync_counter % fft_samples] = fpga_audio_stream;
  }

  if (fpgadata_sync_counter == fpga_sample_rate)
    fpgadata_sync_counter = 0;
}

inline void FFT::digitalWriteDirect(int pin, boolean val) {
  if (val) g_APinDescription[pin].pPort -> PIO_SODR = g_APinDescription[pin].ulPin;
  else    g_APinDescription[pin].pPort -> PIO_CODR = g_APinDescription[pin].ulPin;
}

inline int FFT::digitalReadDirect(int pin) {
  return !!(g_APinDescription[pin].pPort -> PIO_PDSR & g_APinDescription[pin].ulPin);
}

