#include "Arduino.h"
#include "fpga_uart.h"
#include "trx_manager.h"
#include "functions.h"

int FPGA_UART::uart_timeout = 50; //таймаут запроса по uart в миллисек
String FPGA_UART::UART_answer_string;

void FPGA_UART::Init()
{
  Serial2.begin(57600);
  Serial2.setTimeout(uart_timeout);
}

byte FPGA_UART::setMode(byte new_mode)
{
  //посылаем команду выбора новой моды
  Serial.print("Switching mode to: ");
  Serial.print(new_mode, DEC);
  Serial.print(" ");

  Serial2.write(0x08);
  Serial2.flush();
  Serial2.write(new_mode);
  Serial2.flush();

  byte UART_answer = readUARTByte();
  Serial.print("Answer from FPGA: ");
  Serial.println(UART_answer);
  return UART_answer;
}

byte FPGA_UART::getMode()
{
  //посылаем команду получения текущей частоты
  byte res = 0;
  Serial2.write(0x07);
  Serial2.flush();
  byte UART_answer = readUARTByte();
  Serial.print("Current mode is ");
  Serial.println(UART_answer);
  if(UART_answer>50) return 0;
  return UART_answer;
}

int32_t FPGA_UART::setFrequencyPhrase(int32_t new_freq_phrase)
{
  //посылаем команду смещения текущей частоты
  Serial.print("Switching frequency to: ");
  Serial.print(new_freq_phrase);
  Serial.print(" ");

  Serial2.write(0x06);
  Serial2.flush();
  unsigned char simb = new_freq_phrase >> 24;
  Serial.print(simb, HEX);
  Serial.print(".");
  Serial2.write(simb);
  Serial2.flush();
  simb = (new_freq_phrase & (0xff << 16)) >> 16;
  Serial.print(simb, HEX);
  Serial.print(".");
  Serial2.write(simb);
  Serial2.flush();
  simb = (new_freq_phrase & (0xff << 8)) >> 8;
  Serial.print(simb, HEX);
  Serial.print(".");
  Serial2.write(simb);
  Serial2.flush();
  simb = new_freq_phrase & 0xff;
  Serial.print(simb, HEX);
  Serial.print(" ");
  Serial.print(Functions::getFrequencyFromPhrase(new_freq_phrase));
  Serial.print(" ");
  Serial2.write(simb);
  Serial2.flush();
  UART_answer_string = readUARTString();
  Serial.print("Answer from FPGA: ");
  Serial.println(UART_answer_string);
  return Functions::HexStringToInt(UART_answer_string);
}

int32_t FPGA_UART::getFrequencyPhrase()
{
  //посылаем команду получения текущей частоты
  int32_t res = 0;
  Serial2.write(0x05);
  Serial2.flush();
  UART_answer_string = readUARTString();
  res = Functions::HexStringToInt(UART_answer_string);
  Serial.print("Current freq is HEX: ");
  Serial.print(UART_answer_string);
  Serial.print(" PHRASE: ");
  Serial.print(res);
  Serial.print(" FREQ: ");
  Serial.println(Functions::getFrequencyFromPhrase(res));
  if(res<100) return 595591; //7 100 000
  return res;
}

byte FPGA_UART::readUARTByte() //считываем байт из UART
{
  long starttime = millis();
  while (Serial2.available() == 0 && (millis() - starttime < uart_timeout)) { }
  byte res = Serial2.read();
  return res;
}

String FPGA_UART::readUARTString() //считываем строку из UART до символа \n (не включительно)
{
  long starttime = millis();
  String inString = "";
  while (Serial2.available() == 0 && (millis() - starttime < uart_timeout)) { }
  char inChar = (char)Serial2.read();
  while (inChar != '\n') {
    if (millis() - starttime > uart_timeout) return "TIMEOUT";
    if (Serial2.available() > 0)
    {
      inString += (char)inChar;
      inChar = Serial2.read();
    }
  }
  return inString;
}
