#include <SPI.h>
#include "Adafruit_GFX.h"
#include <MCUFRIEND_kbv.h>
#include <TouchScreen.h>
#include "Arduino.h"
#include "lcd.h"
#include "trx_manager.h"
#include "encoder.h"
#include "functions.h"

#define BLACK   0x0000
#define BLUE    0x001F
#define RED     0xF800
#define GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE   0xFFFF
//tft.color565(0, green, 0)

//пины тач-пада
#define YP A1  // must be an analog pin, use "An" notation!
#define XM A2  // must be an analog pin, use "An" notation!
#define YM 7   // can be a digital pin
#define XP 6   // can be a digital pin

MCUFRIEND_kbv LCD::tft(A3, A2, A1, A0, A4);
String LCD::freq_string_hz;
String LCD::freq_string_khz;
String LCD::freq_string_mhz;
bool LCD::bandMenuOpened = false;

void LCD::Init()
{
  //MY LCD ID 0x5408
  resetTouchpadPins();
  uint16_t ID = tft.readID();
  tft.begin(ID);
  tft.setRotation(1);
  tft.fillScreen(BLACK);

  displayFreqInfo();
  displayTopButtons(true);
}

void LCD::displayTopButtons(bool redraw) { //вывод верхних кнопок
  resetTouchpadPins();
  if (redraw) tft.fillRect(0, 0, 319, 65, BLACK);
  int color = CYAN;

  //вывод мод
  if (!bandMenuOpened)
  {
    tft.setTextSize(2);
    if (TRX_Manager::getMode() == 0) color = YELLOW;
    tft.fillRoundRect(5, 5, 70, 60, 10, color);
    tft.setCursor(23, 30);
    tft.setTextColor(BLUE, color);
    tft.print("LSB");

    color = CYAN;
    if (TRX_Manager::getMode() == 1) color = YELLOW;
    tft.fillRoundRect(80, 5, 70, 60, 10, color);
    tft.setCursor(98, 30);
    tft.setTextColor(BLUE, color);
    tft.print("USB");

    color = CYAN;
    tft.fillRoundRect(245, 5, 70, 60, 10, color);
    tft.setCursor(257, 30);
    tft.setTextColor(BLUE, color);
    tft.print("BAND");
  }

  //вывод диапазонов
  if (bandMenuOpened)
  {
    int32_t freq_mhz = (int32_t)(TRX_Manager::getFrequency() / 1000000);
    tft.setTextSize(2);
    color = CYAN; if (freq_mhz == 1) color = YELLOW;
    tft.fillRoundRect(5, 5, 58, 27, 3, color); //1.8
    tft.setCursor(16, 12); tft.setTextColor(BLUE, color); tft.print("1.8");
    color = CYAN; if (freq_mhz == 3) color = YELLOW;
    tft.fillRoundRect(68, 5, 58, 27, 3, color); //3.5
    tft.setCursor(79, 12); tft.setTextColor(BLUE, color); tft.print("3.5");
    color = CYAN; if (freq_mhz == 7) color = YELLOW;
    tft.fillRoundRect(131, 5, 58, 27, 3, color); //7
    tft.setCursor(155, 12); tft.setTextColor(BLUE, color); tft.print("7");
    color = CYAN; if (freq_mhz == 10) color = YELLOW;
    tft.fillRoundRect(194, 5, 58, 27, 3, color); //10
    tft.setCursor(210, 12); tft.setTextColor(BLUE, color); tft.print("10");
    color = CYAN;
    tft.fillRoundRect(257, 5, 58, 27, 3, color); //BACK
    tft.setCursor(262, 12); tft.setTextColor(BLUE, color); tft.print("BACK");

    color = CYAN; if (freq_mhz == 14) color = YELLOW;
    tft.fillRoundRect(5, 37, 58, 27, 3, color); //14
    tft.setCursor(21, 44); tft.setTextColor(BLUE, color); tft.print("14");
    color = CYAN; if (freq_mhz == 18) color = YELLOW;
    tft.fillRoundRect(68, 37, 58, 27, 3, color); //18
    tft.setCursor(84, 44); tft.setTextColor(BLUE, color); tft.print("18");
    color = CYAN; if (freq_mhz == 21) color = YELLOW;
    tft.fillRoundRect(131, 37, 58, 27, 3, color); //21
    tft.setCursor(147, 44); tft.setTextColor(BLUE, color); tft.print("21");
    color = CYAN; if (freq_mhz == 24) color = YELLOW;
    tft.fillRoundRect(194, 37, 58, 27, 3, color); //24
    tft.setCursor(210, 44); tft.setTextColor(BLUE, color); tft.print("24");
    color = CYAN; if (freq_mhz == 28) color = YELLOW;
    tft.fillRoundRect(257, 37, 58, 27, 3, color); //28
    tft.setCursor(273, 44); tft.setTextColor(BLUE, color); tft.print("28");
  }
}

void LCD::displayFreqInfo() { //вывод частоты на экран
  resetTouchpadPins();
  //закрашиваем лишнее на экране если разрядов не хватает
  if (TRX_Manager::getFrequency() < 10000000)
  {
    tft.fillRect(0, 80, 30, 40, BLACK);
    tft.fillRect(300, 80, 20, 40, BLACK);
  }

  tft.setCursor(30, 80);
  if (TRX_Manager::getFrequency() > 10000000) tft.setCursor(10, 80);
  tft.setTextColor(WHITE, BLACK);
  tft.setTextSize(5);

  //добавляем пробелов для вывода частоты
  freq_string_hz = String(TRX_Manager::getFrequency() % 1000);
  freq_string_khz = String((int32_t)(TRX_Manager::getFrequency() / 1000) % 1000);
  freq_string_mhz = String((int32_t)(TRX_Manager::getFrequency() / 1000000) % 1000000);

  if (Encoder::getCurrentIndex() == MENU_FREQ_MHZ) tft.setTextColor(BLACK, WHITE);
  tft.print(freq_string_mhz);
  tft.setTextColor(WHITE, BLACK);
  tft.print(".");

  if (Encoder::getCurrentIndex() == MENU_FREQ_KHZ) tft.setTextColor(BLACK, WHITE);
  tft.print(Functions::addNuls(freq_string_khz));
  tft.setTextColor(WHITE, BLACK);
  tft.print(".");

  if (Encoder::getCurrentIndex() == MENU_FREQ_HZ) tft.setTextColor(BLACK, WHITE);
  tft.print(Functions::addNuls(freq_string_hz));
}

void LCD::displayFFT(byte* data, int data_size)
{
  byte curindex = 0;
  for (int w = 0; w < 320; w++)
  {
    byte curindex = w * data_size / 320;
    tft.drawFastVLine(w, 239 - data[curindex], -100 + data[curindex], BLACK);
    tft.drawFastVLine(w, 239, -data[curindex], BLUE);
  }
}

void LCD::resetTouchpadPins()
{
  //освобождаем пины для экрана
  pinMode(XM, OUTPUT);
  digitalWrite(XM, LOW);
  pinMode(YP, OUTPUT);
  digitalWrite(YP, HIGH);
  pinMode(YM, OUTPUT);
  digitalWrite(YM, LOW);
  pinMode(XP, OUTPUT);
  digitalWrite(XP, HIGH);
}

void LCD::checkTouchPad()
{
  TouchScreen ts = TouchScreen(XP, YP, XM, YM, 530);
  delay(10);
  TSPoint p = ts.getPoint();
  int x = 0;
  int y = 0;

  if (p.z < 1000 && p.z > 10 && p.x < 990) { //ловим нажатие на экран
    x = map(p.x, 110, 880, 0, 319);
    y = map(p.y, 850, 285, 0, 239);
    Serial.print("X = "); Serial.print(p.x);
    Serial.print("\tY = "); Serial.print(p.y);
    Serial.print("\tPressure = "); Serial.print(p.z);
    Serial.print("\tXX = "); Serial.print(x);
    Serial.print("\tYY = "); Serial.print(y);
    Serial.println("");
  }
  while (x > 0 && y > 0 && p.z != 0) //ждём пока отпустят
  {
    delay(10);
    p = ts.getPoint();
  }

  if (!bandMenuOpened)
  {
    if (x >= 5 && x <= 75 && y >= 5 && y <= 65) TRX_Manager::setMode(0); //кнопка LSB
    if (x >= 80 && x <= 150 && y >= 5 && y <= 65) TRX_Manager::setMode(1); //кнопка USB
    if (x >= 245 && x <= 315 && y >= 5 && y <= 65) {
      bandMenuOpened = true;  //кнопка BAND
      displayTopButtons(true);
    }
  }
  else if (bandMenuOpened)
  {
    if (x >= 1 && x <= 63 && y >= 1 && y <= 32) {
      TRX_Manager::setFrequency(1850000);  //кнопка 1.8
      bandMenuOpened = false;
      displayTopButtons(true);
    }
    if (x >= 68 && x <= 126 && y >= 1 && y <= 32) {
      TRX_Manager::setFrequency(3570000);  //кнопка 3.5
      bandMenuOpened = false;
      displayTopButtons(true);
    }
    if (x >= 131 && x <= 189 && y >= 1 && y <= 32) {
      TRX_Manager::setFrequency(7100000);  //кнопка 7
      bandMenuOpened = false;
      displayTopButtons(true);
    }
    if (x >= 194 && x <= 252 && y >= 1 && y <= 32) {
      TRX_Manager::setFrequency(10130000);  //кнопка 10
      bandMenuOpened = false;
      displayTopButtons(true);
    }
    if (x >= 257 && x <= 319 && y >= 1 && y <= 32) { //кнопка BACK
      bandMenuOpened = false;
      displayTopButtons(true);
    }
    if (x >= 1 && x <= 63 && y >= 37 && y <= 64) {
      TRX_Manager::setFrequency(14100000);  //кнопка 14
      bandMenuOpened = false;
      displayTopButtons(true);
    }
    if (x >= 68 && x <= 126 && y >= 37 && y <= 64) {
      TRX_Manager::setFrequency(18100000);  //кнопка 18
      bandMenuOpened = false;
      displayTopButtons(true);
    }
    if (x >= 131 && x <= 189 && y >= 37 && y <= 64) {
      TRX_Manager::setFrequency(21100000);  //кнопка 21
      bandMenuOpened = false;
      displayTopButtons(true);
    }
    if (x >= 194 && x <= 252 && y >= 37 && y <= 64) {
      TRX_Manager::setFrequency(24900000);  //кнопка 24
      bandMenuOpened = false;
      displayTopButtons(true);
    }
    if (x >= 257 && x <= 319 && y >= 37 && y <= 64) {
      TRX_Manager::setFrequency(28100000);  //кнопка 28
      bandMenuOpened = false;
      displayTopButtons(true);
    }
  }

  resetTouchpadPins();
}

