#include "Arduino.h"
#include "trx_manager.h"
#include "fpga_uart.h"
#include "functions.h"
#include "lcd.h"

int32_t TRX_Manager::freq;
int32_t TRX_Manager::freq_phrase; //freq in hz/oscil in hz*2^bits = (freq/50000000)*4194304;
int32_t TRX_Manager::freq_phrase_new;
byte TRX_Manager::mode;
byte TRX_Manager::mode_new;
byte TRX_Manager::mode_to_uart;

void TRX_Manager::Init(int32_t _phrase, byte _mode)
{
  freq = Functions::getFrequencyFromPhrase(_phrase);
  freq_phrase = _phrase; //freq in hz/oscil in hz*2^bits = (freq/50000000)*4194304;
  freq_phrase_new = freq_phrase;
  mode = _mode; //1-USB 0-LSB
  mode_new = mode;
  mode_to_uart = _mode;
  LCD::displayFreqInfo();
  LCD::displayTopButtons(false);
}

void TRX_Manager::setFrequency(int32_t _freq)
{
  if (_freq < 100) _freq = 100;
  if (_freq >= 99999999) _freq = 99999999;
  int32_t old_freq_mhz = ((int32_t)(freq / 1000000) % 1000000);
  freq = _freq;
  freq_phrase_new = round(((float)_freq / 50000000) * 4194304); //freq in hz/oscil in hz*2^bits = (freq/50000000)*4194304;
  if (freq_phrase_new < 0) freq_phrase_new = 0;
  LCD::displayFreqInfo();
  int32_t new_freq_mhz = ((int32_t)(freq / 1000000) % 1000000);
  if (old_freq_mhz != new_freq_mhz)
    LCD::displayTopButtons(false);
}

int32_t TRX_Manager::getFrequency()
{
  return freq;
}

void TRX_Manager::setMode(byte _mode)
{
  mode_new = _mode;
  mode = _mode;
  LCD::displayTopButtons(false);
}

byte TRX_Manager::getMode()
{
  return mode;
}

void TRX_Manager::checkAndSendFreqToFPGA()
{
  if (freq_phrase != freq_phrase_new)
  {
    int32_t freq_from_uart = FPGA_UART::setFrequencyPhrase(freq_phrase_new);
    while (freq_from_uart != freq_phrase_new)
    {
      Serial.print("wrong phase answered ANS: ");
      Serial.print(freq_from_uart);
      Serial.print(" NEED: ");
      Serial.println(freq_phrase_new);
      freq_from_uart = FPGA_UART::setFrequencyPhrase(freq_phrase_new);
    }
    freq_phrase = freq_phrase_new;
  }
  if (mode_to_uart != mode_new)
  {
    int32_t mode_from_uart = FPGA_UART::setMode(mode_new);
    while (mode_from_uart != mode_new)
    {
      Serial.print("wrong mode answered ANS: ");
      Serial.print(mode_from_uart);
      Serial.print(" NEED: ");
      Serial.print(mode_new);
      Serial.print(" PLANNED: ");
      Serial.println(mode_to_uart);
      mode_from_uart = FPGA_UART::setMode(mode_new);
    }
    mode_to_uart = mode_new;
  }
}

