#include <inttypes.h>
#include "encoder.h"
#include "trx_manager.h"
#include "fpga_uart.h"
#include "lcd.h"
#include "fft.h"
#include "SimpleTimer.h"

//подключение энкодера
#define ENC_CLK 47
#define ENC_DT 49
#define ENC_SW 51
#define ENC_PW 53

SimpleTimer timer;

void setup() { //первоначальные настройки
  //запускаем UART для отладки
  Serial.begin(57600);
  Serial.println("Hello Computer");

  delay(1000);
  LCD::Init(); //запускаем LCD экран
  Encoder::Init(ENC_CLK, ENC_DT, ENC_SW, ENC_PW); //запускаем энкодер
  FPGA_UART::Init(); //запускаем соединение с FPGA
  TRX_Manager::Init(FPGA_UART::getFrequencyPhrase(), FPGA_UART::getMode()); //запускаем менеджер трансивера
  FFT::Init(); //запускаем водопад сигнала

  //таймеры
  timer.setInterval(100, TRX_Manager::checkAndSendFreqToFPGA); //проверка соответствия частоты/моды контроллера и FPGA
  timer.setInterval(100, LCD::checkTouchPad); //проверка тачпада
  timer.setInterval(500, FFT::generate_fft); //генератор водопада FFT
}

void loop() { //постоянный цикл
  timer.run();
}

